# Gridlock - a collection of grid-based strategy games
# Copyright (c) 2009 Brian Nenninger (brian@dozingcatsoftware.com)
# Gridlock comes with ABSOLUTELY NO WARRANTY, and is released under version 3 of the GNU GPL.

from GameTypes import GAMES
import PGUtils

HUMAN_PLAYER = 0
AI_PLAYER = 1
NETWORK_PLAYER = 2

class Config(object):
    
    def __init__(self):
        self.game_name = None
        self.variation = None        
        self.game = None
        self.ai = {}
        self.aikey = {}
        self.network = {}
        
    def default_game_key(self):
        return self.game_items()[0][1]
        
    def default_variation_key(self):
        return self.variation_items()[0][1]

    def game_items(self):
        # returns (display_name, name) tuples, display name defaults to name
        return [(g.get('display_name', g['name']), g['name']) for g in GAMES]
        
    def has_game(self, game):
        return game in (g['name'] for g in GAMES)
        
    def has_variation(self, variation):
        return variation in (v['name'] for v in self.selected_game_dict()['variations'])
        
    def has_ai_key(self, aikey):
        return aikey in (a['name'] for a in self.selected_game_dict().get('ai',[]))
        
    def selected_game_dict(self):
        return PGUtils.first_item((g for g in GAMES if self.game_name==g['name']), {})
        
    def selected_game_display_name(self):
        gdict = self.selected_game_dict()
        return gdict.get('display_name', gdict['name'])
        
    def start_game(self):
        import Game
        self.game = Game.Game(self.game_name, self.game_params())
        return self.game
        
    def select_game(self, gname, variation=None):
        if gname is None:
            gname = self.default_game_key()
        if gname!=self.game_name and self.has_game(gname):
            self.game_name = gname
            self.select_variation(variation if variation else self.default_variation_key(), _startgame=False)
            # always clear AIs, clients need to set using set_ai_for_player
            self.ai.clear()
            self.aikey.clear()
            self.start_game()
            return True
        return False
        
    def selected_game_index(self):
        return PGUtils.first_item((i for i,g in enumerate(GAMES) if self.game_name==g['name']), 0)
        
    def variation_items(self):
        return [(v['name'],v['name']) for v in self.selected_game_dict().get('variations',[])]
        
    def select_variation(self, vdata, _startgame=True):
        if self.variation!=vdata and self.has_variation(vdata):
            self.variation = vdata
            if _startgame:
                self.start_game()
            return True
        return False
        
    def selected_variation_index(self):
        return PGUtils.first_item((i for i,v in enumerate(self.variation_items()) if self.variation==v[0]), 0)
        
    def game_params(self):
        gsel = self.selected_game_dict()
        if gsel:
            # if current variation doesn't exist, use first
            vsel = PGUtils.first_item((v for v in gsel['variations'] if self.variation==v['name']))
            if not vsel: vsel = gsel['variations'][0]
            return vsel['params']
        return {}
        
    def ai_items(self):
        return [(a['name'],a['name']) for a in self.selected_game_dict().get('ai',[])]
        
    def ai_params_for_key(self, key):
        gsel = self.selected_game_dict()
        if gsel:
            asel = PGUtils.first_item((a for a in gsel['ai'] if key==a['name']))
            if asel: return asel['params']
        return {}
        
    def selected_ai_index_for_player(self, pnum):
        key = self.aikey[pnum]
        return PGUtils.first_item((i for i,a in enumerate(self.ai_items()) if key==a[0]), -1)

    def ai_class(self):
        gmodule = __import__(self.game_name)
        # check for AI_CLASS attribute, then [game name]AI
        if hasattr(gmodule, 'AI_CLASS'): return gmodule.AI_CLASS
        ai_class = self.game_name + 'AI'
        from GenericAI import GenericAI
        return getattr(gmodule, ai_class, GenericAI)
        
    def set_ai_for_player(self, aikey, pnum):
        if not self.has_ai_key(aikey):
            return False
        self.aikey[pnum] = aikey
        if not aikey:
            self.ai[pnum] = None
        else:
            self.network[pnum] = None
            aiparams = self.ai_params_for_key(aikey)
            self.ai[pnum] = self.ai_class()(aiparams)
        return True
            
    def clear_ais(self):
        self.ai.clear()
        self.aikey.clear()
            
    def set_network_for_player(self, net, pnum):
        self.network[pnum] = net
        if net:
            self.ai[pnum] = None
            self.aikey[pnum] = None
            
    def network_players(self):
        return [(pnum, self.network[pnum]) for pnum in self.network if self.network[pnum] is not None]
        
    def set_human_player(self, pnum):
        self.ai[pnum] = None
        self.aikey[pnum] = None
        self.network[pnum] = None
            
    def current_game_state(self):
        if self.game:
            return self.game.current_state
        return None
            
    def player_type(self, pnum):
        if self.ai.get(pnum): return AI_PLAYER
        if self.network.get(pnum): return NETWORK_PLAYER
        return HUMAN_PLAYER
        
    def current_player(self):
        return self.current_game_state().current_player
        
    def current_player_type(self):
        return self.player_type(self.current_player())

    def current_ai(self):
        return self.ai.get(self.current_player())
        
    def current_network(self):
        return self.network.get(self.current_player())
                    
    def serialize(self):
        return {'game': self.game_name, 'variation': self.variation}
        
    def update_from_dict(self, d):
        self.game_name = d['game']
        self.variation = d['variation']
        
def deserialize_config(d):
    c = Config()
    c.game_name = d['game']
    c.variation = d['variation']
    return c
    